use futures::Future;
use nostr_rs_client::{Error as ClientError, Event, Pool};
use nostr_rs_storage::{RocksDb, Storage};
use nostr_rs_types::{client::Subscribe, Request, Response};
use std::pin::Pin;
use tokio::sync::mpsc;

#[derive(Debug, thiserror::Error)]
pub enum Error {
    #[error("Nostr: {0}")]
    Addr(#[from] nostr_rs_types::types::addr::Error),

    #[error("client: {0}")]
    Client(#[from] ClientError),
}

fn on_connection(
    host: &str,
    _socket: mpsc::Sender<Request>,
) -> Pin<Box<dyn Future<Output = ()> + Send>> {
    println!("Reconnecting to {}", host);
    Box::pin(async move {})
}

#[tokio::main]
async fn main() {
    env_logger::init();
    let mut clients = Pool::new();
    clients.send(Subscribe::default().into()).await;

    let _ = clients
        .connect_to("wss://relay.damus.io/", u16::MAX, Some(on_connection))
        .await;
    let _ = clients
        .connect_to("wss://brb.io", u16::MAX, Some(on_connection))
        .await;
    let _ = clients
        .connect_to("wss://nos.lol", u16::MAX, Some(on_connection))
        .await;
    let _ = clients
        .connect_to("wss://relay.current.fyi", u16::MAX, Some(on_connection))
        .await;
    let _ = clients
        .connect_to("wss://eden.nostr.land", u16::MAX, Some(on_connection))
        .await;
    let _ = clients
        .connect_to("wss://relay.snort.social", u16::MAX, Some(on_connection))
        .await;
    let db = RocksDb::new("./db").expect("db");

    loop {
        if let Some((msg, _relayed_by)) = clients.recv().await {
            match msg {
                Event::Response(Response::Event(x)) => {
                    let event = x.event;
                    db.store(&event).expect("valid");
                }
                Event::Response(Response::EndOfStoredEvents(_)) => {}
                msg => {
                    panic!("{:?}", msg);
                }
            }
        }
    }
}
